"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ReactOnRails_client_1 = require("../ReactOnRails.client");
var RSCClientRoot_1 = require("../RSCClientRoot");
/**
 * Registers React Server Components (RSC) with React on Rails.
 * This function wraps server components with RSCClientRoot to handle client-side rendering
 * and hydration of RSC payloads.
 *
 * Important: Components registered with this function are not included in the client bundle.
 * Instead, when the component needs to be rendered:
 * 1. A request is made to `${rscPayloadGenerationUrlPath}/${componentName}`
 * 2. The server returns an RSC payload containing:
 *    - Server component rendered output
 *    - References to client components that need hydration
 *    - Data props passed to client components
 * 3. The RSC payload is processed and rendered as HTML on the client
 *
 * This approach enables:
 * - Smaller client bundles (server components code stays on server)
 * - Progressive loading of components
 * - Automatic handling of data requirements
 *
 * @param options - Configuration options for RSC rendering
 * @param options.rscPayloadGenerationUrlPath - The base URL path where RSC payloads will be fetched from
 * @param componentNames - Names of server components to register
 *
 * @example
 * ```js
 * registerServerComponent({
 *   rscPayloadGenerationUrlPath: '/rsc_payload'
 * }, 'ServerComponent1', 'ServerComponent2');
 *
 * // When ServerComponent1 renders, it will fetch from: /rsc_payload/ServerComponent1
 * ```
 */
var registerServerComponent = function (options) {
    var componentNames = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        componentNames[_i - 1] = arguments[_i];
    }
    var componentsWrappedInRSCClientRoot = {};
    var _loop_1 = function (name_1) {
        componentsWrappedInRSCClientRoot[name_1] = function (componentProps, _railsContext, domNodeId) { return (0, RSCClientRoot_1.default)({
            componentName: name_1,
            rscPayloadGenerationUrlPath: options.rscPayloadGenerationUrlPath,
            componentProps: componentProps,
        }, _railsContext, domNodeId); };
    };
    for (var _a = 0, componentNames_1 = componentNames; _a < componentNames_1.length; _a++) {
        var name_1 = componentNames_1[_a];
        _loop_1(name_1);
    }
    ReactOnRails_client_1.default.register(componentsWrappedInRSCClientRoot);
};
exports.default = registerServerComponent;
