"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.streamServerRenderedComponent = exports.transformRenderStreamChunksToResultObject = void 0;
var ReactDOMServer = require("react-dom/server");
var stream_1 = require("stream");
var ComponentRegistry_1 = require("./ComponentRegistry");
var createReactOutput_1 = require("./createReactOutput");
var isServerRenderResult_1 = require("./isServerRenderResult");
var buildConsoleReplay_1 = require("./buildConsoleReplay");
var handleError_1 = require("./handleError");
var serverRenderUtils_1 = require("./serverRenderUtils");
var stringToStream = function (str) {
    var stream = new stream_1.PassThrough();
    stream.write(str);
    stream.end();
    return stream;
};
/**
 * Creates a new Readable stream that safely buffers all events from the input stream until reading begins.
 *
 * This function solves two important problems:
 * 1. Error handling: If an error occurs on the source stream before error listeners are attached,
 *    it would normally crash the process. This wrapper buffers error events until reading begins,
 *    ensuring errors are properly handled once listeners are ready.
 * 2. Event ordering: All events (data, error, end) are buffered and replayed in the exact order
 *    they were received, maintaining the correct sequence even if events occur before reading starts.
 *
 * @param stream - The source Readable stream to buffer
 * @returns {Object} An object containing:
 *   - stream: A new Readable stream that will buffer and replay all events
 *   - emitError: A function to manually emit errors into the stream
 */
var bufferStream = function (stream) {
    var bufferedEvents = [];
    var startedReading = false;
    var listeners = ['data', 'error', 'end'].map(function (event) {
        var listener = function (data) {
            if (!startedReading) {
                bufferedEvents.push({ event: event, data: data });
            }
        };
        stream.on(event, listener);
        return { event: event, listener: listener };
    });
    var bufferedStream = new stream_1.Readable({
        read: function () {
            var _this = this;
            if (startedReading)
                return;
            startedReading = true;
            // Remove initial listeners
            listeners.forEach(function (_a) {
                var event = _a.event, listener = _a.listener;
                return stream.off(event, listener);
            });
            var handleEvent = function (_a) {
                var event = _a.event, data = _a.data;
                if (event === 'data') {
                    _this.push(data);
                }
                else if (event === 'error') {
                    _this.emit('error', data);
                }
                else {
                    _this.push(null);
                }
            };
            // Replay buffered events
            bufferedEvents.forEach(handleEvent);
            // Attach new listeners for future events
            ['data', 'error', 'end'].forEach(function (event) {
                stream.on(event, function (data) { return handleEvent({ event: event, data: data }); });
            });
        }
    });
    return {
        stream: bufferedStream,
        emitError: function (error) {
            if (startedReading) {
                bufferedStream.emit('error', error);
            }
            else {
                bufferedEvents.push({ event: 'error', data: error });
            }
        },
    };
};
var transformRenderStreamChunksToResultObject = function (renderState) {
    var consoleHistory = console.history;
    var previouslyReplayedConsoleMessages = 0;
    var transformStream = new stream_1.PassThrough({
        transform: function (chunk, _, callback) {
            var htmlChunk = chunk.toString();
            var consoleReplayScript = (0, buildConsoleReplay_1.default)(consoleHistory, previouslyReplayedConsoleMessages);
            previouslyReplayedConsoleMessages = (consoleHistory === null || consoleHistory === void 0 ? void 0 : consoleHistory.length) || 0;
            var jsonChunk = JSON.stringify((0, serverRenderUtils_1.createResultObject)(htmlChunk, consoleReplayScript, renderState));
            this.push("".concat(jsonChunk, "\n"));
            callback();
        }
    });
    var pipedStream = null;
    var pipeToTransform = function (pipeableStream) {
        pipeableStream.pipe(transformStream);
        pipedStream = pipeableStream;
    };
    // We need to wrap the transformStream in a Readable stream to properly handle errors:
    // 1. If we returned transformStream directly, we couldn't emit errors into it externally
    // 2. If an error is emitted into the transformStream, it would cause the render to fail
    // 3. By wrapping in Readable.from(), we can explicitly emit errors into the readableStream without affecting the transformStream
    // Note: Readable.from can merge multiple chunks into a single chunk, so we need to ensure that we can separate them later
    var _a = bufferStream(transformStream), readableStream = _a.stream, emitError = _a.emitError;
    var writeChunk = function (chunk) { return transformStream.write(chunk); };
    var endStream = function () {
        transformStream.end();
        pipedStream === null || pipedStream === void 0 ? void 0 : pipedStream.abort();
    };
    return { readableStream: readableStream, pipeToTransform: pipeToTransform, writeChunk: writeChunk, emitError: emitError, endStream: endStream };
};
exports.transformRenderStreamChunksToResultObject = transformRenderStreamChunksToResultObject;
var streamRenderReactComponent = function (reactRenderingResult, options) {
    var componentName = options.name, throwJsErrors = options.throwJsErrors, domNodeId = options.domNodeId;
    var renderState = {
        result: null,
        hasErrors: false,
        isShellReady: false
    };
    var _a = (0, exports.transformRenderStreamChunksToResultObject)(renderState), readableStream = _a.readableStream, pipeToTransform = _a.pipeToTransform, writeChunk = _a.writeChunk, emitError = _a.emitError, endStream = _a.endStream;
    var renderingStream = ReactDOMServer.renderToPipeableStream(reactRenderingResult, {
        onShellError: function (e) {
            var error = (0, serverRenderUtils_1.convertToError)(e);
            renderState.hasErrors = true;
            renderState.error = error;
            if (throwJsErrors) {
                emitError(error);
            }
            var errorHtml = (0, handleError_1.default)({ e: error, name: componentName, serverSide: true });
            writeChunk(errorHtml);
            endStream();
        },
        onShellReady: function () {
            renderState.isShellReady = true;
            pipeToTransform(renderingStream);
        },
        onError: function (e) {
            if (!renderState.isShellReady) {
                return;
            }
            var error = (0, serverRenderUtils_1.convertToError)(e);
            if (throwJsErrors) {
                emitError(error);
            }
            renderState.hasErrors = true;
            renderState.error = error;
        },
        identifierPrefix: domNodeId,
    });
    return readableStream;
};
var streamServerRenderedComponent = function (options, renderStrategy) {
    var componentName = options.name, domNodeId = options.domNodeId, trace = options.trace, props = options.props, railsContext = options.railsContext, throwJsErrors = options.throwJsErrors;
    try {
        var componentObj = ComponentRegistry_1.default.get(componentName);
        (0, serverRenderUtils_1.validateComponent)(componentObj, componentName);
        var reactRenderingResult = (0, createReactOutput_1.default)({
            componentObj: componentObj,
            domNodeId: domNodeId,
            trace: trace,
            props: props,
            railsContext: railsContext,
        });
        if ((0, isServerRenderResult_1.isServerRenderHash)(reactRenderingResult) || (0, isServerRenderResult_1.isPromise)(reactRenderingResult)) {
            throw new Error('Server rendering of streams is not supported for server render hashes or promises.');
        }
        return renderStrategy(reactRenderingResult, options);
    }
    catch (e) {
        if (throwJsErrors) {
            throw e;
        }
        var error = (0, serverRenderUtils_1.convertToError)(e);
        var htmlResult = (0, handleError_1.default)({ e: error, name: componentName, serverSide: true });
        var jsonResult = JSON.stringify((0, serverRenderUtils_1.createResultObject)(htmlResult, (0, buildConsoleReplay_1.default)(), { hasErrors: true, error: error, result: null }));
        return stringToStream(jsonResult);
    }
};
exports.streamServerRenderedComponent = streamServerRenderedComponent;
var streamServerRenderedReactComponent = function (options) { return (0, exports.streamServerRenderedComponent)(options, streamRenderReactComponent); };
exports.default = streamServerRenderedReactComponent;
